@@ -1,7 +1,7 @@
 package Carmel;
 use strict;
 use 5.012000;
-use version; our $VERSION = version->declare('v0.1.28');
+use version; our $VERSION = version->declare('v0.1.29');
 
 1;
 __END__
@@ -29,6 +29,9 @@ Carmel - CPAN Artifact Repository Manager
   # show a location where a module is installed
   carmel show Plack
 
+  # update snapshot
+  carmel update
+
   # Runs your perl script with modules from artifacts
   carmel exec perl ...
 
@@ -105,11 +108,6 @@ Instead, if you have:
 it will find the latest that satisfies the (empty) requirement, which
 is in C<URI-1.64/blib/lib>.
 
-The fact that it prefers the latest, rather than the oldest, might
-change in the future once a mechanism to make snapshot is introduced,
-since you will not like to upgrade one of the dependencies
-unexpectedly.
-
 You have a choice to execute a subprocess from Carmel, by using the
 C<exec> sub command. If you prefer a fine grained control, you can
 also use C<env> or C<export> command to integrate with your own shell
@@ -117,10 +115,12 @@ script wrapper.
 
 =head2 SNAPSHOT SUPPORT
 
-As of v0.1.26, Carmel supports I<saving> snapshot file in
-C<cpanfile.snapshot>, in a compatible format with L<Carton>. Support
-for restoring the versions recorded in the snapshot file will be added
-in the future version.
+As of v0.1.29, Carmel supports saving and loading snapshot file in
+C<cpanfile.snapshot>, in a compatible format with L<Carton>. Versions
+saved in the snapshot file will be preserved across multiple runs of
+Carmel across machines, so that versions frozen in one environment can
+be committed to a source code repository, and can be reproduce in
+another box, so long as the perl version and architecture is the same.
 
 =head1 COMMUNITY
 
