@@ -94,17 +94,34 @@ sub cmd_inject {
     $self->install("--reinstall", @args);
 }
 
+sub cmd_update {
+    my($self, @args) = @_;
+
+    die "Usage: carmel update\n" if @args; # TODO supprot args
+
+    # reinstall everything in cpanfile to build artifacts
+    my $cpanfile = $self->try_cpanfile
+      or Carp::croak "Can't locate 'cpanfile' to load module list.";
+    $self->install_with_cpanfile(Module::CPANfile->load($cpanfile));
+
+    # then rebuild the snapshot
+    my @artifacts = $self->install_from_cpanfile(1);
+    $self->dump_bootstrap(\@artifacts);
+    $self->save_snapshot(\@artifacts);
+}
+
 sub cmd_install {
     my($self, @args) = @_;
 
     die "Usage: carmel install\n" if @args;
 
-    $self->install_from_cpanfile;
-    $self->post_install;
+    my @artifacts = $self->install_from_cpanfile;
+    $self->dump_bootstrap(\@artifacts);
+    $self->save_snapshot(\@artifacts);
 }
 
 sub install_from_cpanfile {
-    my $self = shift;
+    my($self, $no_snapshot) = @_;
 
     my $requirements = CPAN::Meta::Requirements->new;
     $self->resolve(
@@ -117,10 +134,12 @@ sub install_from_cpanfile {
             if ($dist) {
                 # TODO pass $dist->distfile to cpanfile
                 my $ver = $dist->version_for($module) || '0';
-                $want_version = "== $ver";
+                $want_version = $ver ? "== $ver" : $ver;
             }
             $requirements->add_string_requirement($module => $want_version);
         },
+        1, # strict
+        $no_snapshot,
     );
 
     if (my @missing = $requirements->required_modules) {
@@ -134,12 +153,14 @@ sub install_from_cpanfile {
     }
 
     my @artifacts;
-    $self->resolve(sub { push @artifacts, $_[0] });
+    $self->resolve(sub { push @artifacts, $_[0] }, undef, 0, $no_snapshot);
 
     # $self->requirements has been upgraded at this point with the whole subreqs
     printf "---> Complete! %d cpanfile dependencies. %d modules installed.\n" .
       "---> Use `carmel show [module]` to see where a module is installed.\n",
       scalar(grep { $_ ne 'perl' } $self->build_requirements->required_modules), scalar(@artifacts);
+
+    return @artifacts;
 }
 
 sub is_core {
@@ -182,6 +203,8 @@ sub install {
         next unless $ent->is_dir && $ent->child("blib/meta/install.json")->exists;
         $self->repo->import_artifact($ent);
     }
+
+    $self->repository_base->child('perl5')->remove_tree({ safe => 0 });
 }
 
 sub quote {
@@ -212,16 +235,6 @@ sub transform {
     }
 }
 
-sub post_install {
-    my $self = shift;
-
-    my @artifacts;
-    $self->resolve(sub { push @artifacts, $_[0] });
-
-    $self->dump_bootstrap(\@artifacts);
-    $self->save_snapshot(\@artifacts);
-}
-
 sub save_snapshot {
     my($self, $artifacts) = @_;
 
@@ -368,16 +381,24 @@ sub cmd_tree {
 sub cmd_rollout {
     my $self = shift;
 
-    require ExtUtils::Install;
-    require ExtUtils::InstallPaths;
-
     my @artifacts;
     $self->resolve(sub { push @artifacts, $_[0] });
 
     my $install_base = Path::Tiny->new("local")->absolute;
     $install_base->remove_tree({ safe => 0 }) if $install_base->exists;
 
-    for my $artifact (@artifacts) {
+    $self->rollout_to($install_base, \@artifacts);
+
+    Path::Tiny->new("local/.carmel")->touch;
+}
+
+sub rollout_to {
+    my($self, $install_base, $artifacts) = @_;
+
+    require ExtUtils::Install;
+    require ExtUtils::InstallPaths;
+
+    for my $artifact (@$artifacts) {
         my $dir = pushd $artifact->path;
 
         my $paths = ExtUtils::InstallPaths->new(install_base => $install_base);
@@ -401,8 +422,6 @@ sub cmd_rollout {
 
         select $old unless $self->verbose;
     }
-
-    Path::Tiny->new("local/.carmel")->touch;
 }
 
 sub cmd_package {
@@ -552,7 +571,7 @@ sub merge_requirements {
 }
 
 sub resolve_recursive {
-    my($self, $root_reqs, $requirements, $snapshot, $seen, $cb, $missing_cb, $depth) = @_;
+    my($self, $root_reqs, $requirements, $snapshot, $seen, $cb, $missing_cb, $strict, $depth) = @_;
 
     # TODO rather than mutating $root_reqs directly, we should create a new object
     # that allows accessing the result $requirements
@@ -563,16 +582,15 @@ sub resolve_recursive {
 
         my $artifact;
         my $dist;
-        if ($dist = $self->find_in_snapshot($snapshot, $module, $want_version)) {
-            my $version = $dist->version_for($module) || '0';
-            $artifact = $self->repo->find_exact($module, "== $version",
-                                                sub { $self->accepts_all($_[0], $root_reqs, $dist) });
+        if ($dist = $self->find_in_snapshot($snapshot, $module, $root_reqs)) {
+            $artifact = $self->repo->find_match($module, sub { $_[0]->distname eq $dist->name });
+            $artifact ||= $self->repo->find($module, $want_version) unless $strict;
         } elsif ($self->is_core($module, $want_version)) {
             next;
+        } else {
+            $artifact = $self->repo->find($module, $want_version);
         }
 
-        $artifact ||= $self->repo->find($module, $want_version);
-
         # FIXME there's a chance different version of the same module can be loaded here
         if ($artifact) {
             warn sprintf "   %s (%s) in %s\n", $module, $artifact->version_for($module), $artifact->path if $self->verbose;
@@ -582,7 +600,7 @@ sub resolve_recursive {
             my $reqs = $artifact->requirements;
             $self->merge_requirements($root_reqs, $reqs, $artifact->distname);
 
-            $self->resolve_recursive($root_reqs, $reqs, $snapshot, $seen, $cb, $missing_cb, $depth + 1);
+            $self->resolve_recursive($root_reqs, $reqs, $snapshot, $seen, $cb, $missing_cb, $strict, $depth + 1);
         } else {
             $missing_cb->($module, $want_version, $dist, $depth);
         }
@@ -590,42 +608,43 @@ sub resolve_recursive {
 }
 
 sub resolve {
-    my($self, $cb, $missing_cb) = @_;
+    my($self, $cb, $missing_cb, $strict, $no_snapshot) = @_;
     $missing_cb ||= sub {
         my($module, $want_version, $dist, $depth) = @_;
         die "Can't find an artifact for $module => $want_version\n" .
             "You need to run `carmel install` first to get the modules installed and artifacts built.\n";
     };
-    $self->resolve_recursive($self->requirements, $self->requirements->clone, $self->snapshot,
-                             {}, $cb, $missing_cb, 0);
+
+    my $snapshot = $no_snapshot ? undef : $self->snapshot;
+
+    $self->resolve_recursive($self->requirements, $self->requirements->clone, $snapshot,
+                             {}, $cb, $missing_cb, $strict, 0);
 }
 
 sub find_in_snapshot {
-    my($self, $snapshot, $module, $want_version) = @_;
+    my($self, $snapshot, $module, $reqs) = @_;
 
     return unless $snapshot;
 
     if (my $dist = $snapshot->find($module)) {
-        my $version = $dist->version_for($module);
-        if ($self->accepts($module, $want_version, $version)) {
-            $version = 'undef' unless defined $version;
-            warn "Found $module $version in snapshot: satisfies $want_version\n" if $self->verbose;
+        warn "@{[$dist->name]} found in snapshot for $module\n" if $self->verbose;
+        if ($self->accepts_all($reqs, $dist)) {
             return $dist;
         }
     }
 
+    warn "$module not found in snapshot\n" if $self->verbose;
+
     return;
 }
 
 sub accepts_all {
-    my($self, $artifact, $reqs, $dist) = @_;
+    my($self, $reqs, $dist) = @_;
 
-    return unless $artifact->distname eq $dist->name;
-
-    my @packages = keys %{$artifact->provides};
+    my @packages = keys %{$dist->provides};
 
     for my $pkg (@packages) {
-        my $version = $artifact->provides->{$pkg}{version} || '0';
+        my $version = $dist->provides->{$pkg}{version} || '0';
         return unless $reqs->accepts_module($pkg, $version);
     }
 
@@ -650,7 +669,7 @@ sub snapshot {
         return $snapshot;
     }
 
-    return undef;
+    return;
 }
 
 1;
